<CsoundSynthesizer>
<CsOptions>
-odac
</CsOptions>
<CsInstruments>
/*
	basic demonstration of parsing a json string containing score directives for specified instruments, and then scheduling them accordingly

*/
sr = 44100
kr = 4410
nchnls = 2
0dbfs = 10
seed 0


#include "json.udo"

; the json string to parse
gSjson = {{

	{"instruments": {

		"test_1": [
			[0, 2, 440],
			[2, 2, 880],
			[4, 2, 220]
		],

		"test_2": [
			[6, 2, 880],
			[8, 2, 440]
		],

		"empty": []
	}

}}


; waves for oscillators in test instruments
gifnSquare ftgen 0, 0, 4096, 10, 1, 0 , .33, 0, .2 , 0, .14, 0 , .11, 0, .09 
gifnSaw ftgen 0, 0, 4096, 10, 0, .2, 0, .4, 0, .6, 0, .8, 0, 1, 0, .8, 0, .6, 0, .4, 0, .2


/*
	test instrument 1
*/
instr test_1
	ifreq = p4
	kamp linseg 1, p3, 0
	aout oscil kamp, ifreq, gifnSquare
	outs aout, aout
endin


/*
	test instrument 2
*/
instr test_2
	ifreq = p4
	kamp linseg 0, p3, 1
	aout oscil kamp, ifreq, gifnSaw
	outs aout, aout
endin


/*
	parsing and scheduling
*/
instr boot

	; get the object (as string) with key "instruments" 
	itype, SjsonInstruments, i_ json_parse gSjson, "instruments"

	; if array of instruments is found
	if (itype == 1) then
		
		; get the instrument names/keys and loop through
		Sinstruments[] json_getkeys SjsonInstruments
		indexinstr = 0
		while (indexinstr < lenarray(Sinstruments)) do

			; get the array containing score directives
			itype, SjsonOrcItems, i_ json_parse gSjson, Sinstruments[indexinstr]

			; only continue if an array is actually returned
			if (itype == 2) then

				; go through each score entry in the array
				indexorcitem = 0
				while (indexorcitem < json_arraylength(SjsonOrcItems)) do

					; get the array of p fields for this score entry
					itype, SjsonPfields, i_ json_parsearray SjsonOrcItems, indexorcitem
				

					; this could be looped through if number unknown, but would also be nicer to get these all back in one go as an array
					i_, S_, ip2 json_parsearray SjsonPfields, 0
					i_, S_, ip3 json_parsearray SjsonPfields, 1
					i_, S_, ip4 json_parsearray SjsonPfields, 2

					; print what's happening and do it
					Scoreline = sprintf("i\"%s\" %d %d %d\n", Sinstruments[indexinstr], ip2, ip3, ip4)
					prints Scoreline
					scoreline_i Scoreline

					indexorcitem += 1
				od

			endif

			indexinstr += 1
		od

	endif

endin


</CsInstruments>
<CsScore>
f1 0 16384 10 1                                          ; Sine
f2 0 16384 10 1 0.5 0.3 0.25 0.2 0.167 0.14 0.125 .111   ; Sawtooth
f3 0 16384 10 1 0   0.3 0    0.2 0     0.14 0     .111   ; Square

i"boot" 0 10

</CsScore>
</CsoundSynthesizer>